/**
* Activity.js
*
* @description :: activity controller imported from localhost MySql server at 18/3/2020 16:24:20.
* @docs        :: http://sailsjs.org/#!documentation/controllers
* @routes      :: /* Activity /*
*              'get /activity/getAllActivity/': 'ActivityController.getAllActivity',
*              'get /activity/getActivityById/:id/': 'ActivityController.getActivityById',
*              'post /activity/createActivity/': 'ActivityController.createActivity', 
*              'put /activity/updateActivity/': 'ActivityController.updateActivity', 
*              'detele /activity/deleteActivityById/:id/': 'ActivityController.deleteActivityById',
* 
*/


module.exports = {

    
    getAllActivity: async function (req, res) {
        let data = await Activity.find().populate('idActivityGroup').sort('id DESC');
        return res.ok(data);  
    },

    getSalesmanActivities: async function(req,res){
        let thisMonth= new Date().toLocaleDateString("pt-BR").split("-")[1];
        let thisYear= new Date().toLocaleDateString("pt-BR").split("-")[0];
        let activities = await Activity.find();
        let groups = await Activitygroup.find();
        let info=[];
        let salesman = await User.findOne({id:req.user.id}).populate('idTeam');
        if(salesman){
            for(let g of groups){
                let grp = {};
                grp.name = g.name;
                filteredActivities = activities.filter(element =>{
                    return element.idActivityGroup == g.id;
                })
                grp.totalQtd = 0;
                grp.totalDone = 0;
                grp.activity = [];
                for(let activity of filteredActivities){
                    let act = {};
                    grp.totalQtd += activity.monthLimit;
                    act.activity = activity;
                    act.qtd = 0;
                    let registersByActivity = await Activityuser.find({idUser: salesman.id, idActivity: activity.id}).populate('idUser').populate('idUserParent').populate('idActivity'); 
                    for(let r of registersByActivity){
                        r.month = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                        r.year = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                        if(r.month == thisMonth && r.year == thisYear){
                            grp.totalDone++;
                            act.qtd++;
                        }              
                    }
                    grp.activity.push(act);
                } 
                info.push(grp);
            }
            for(let i of info){
                i.totalCompletion = Math.trunc((i.totalDone/i.totalQtd)*100);
            }
            info.sort((a,b)=> {return b.totalCompletion-a.totalCompletion;})
            res.ok(info)
        }else{
            return res.status(500).send({
                success: false,
                message: "Dados não encontrados."
            });
        }
    },

    

    getNextActivities: async function(req,res){
        const thisDay = parseInt(new Date().toLocaleDateString("pt-BR").split("-")[2]);
        let thisMonth= new Date().toLocaleDateString("pt-BR").split("-")[1];
        let thisYear= new Date().toLocaleDateString("pt-BR").split("-")[0];
        const limitDay = parseInt(thisDay) + 7;
        let activities = await Activity.find();
        activities = activities.filter(element=>{
            return element.limit >= thisDay && element.limit <= limitDay;
        })
        let info=[];
        let salesman = await User.findOne({id:req.user.id}).populate('idTeam');
        if(salesman){
            for(let activity of activities){
                let act = {};
                act.activity = activity;
                let registersByActivity = await Activityuser.find({idUser: salesman.id, idActivity: activity.id}).populate('idUser').populate('idUserParent').populate('idActivity');
                let qtd = 0;
                for(let r of registersByActivity){
                    r.month = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                    r.year = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                    if(r.month == thisMonth && r.year == thisYear){
                        qtd++;
                    }              
                }
                act.limit = activity.monthLimit;
                act.qtdDone = qtd;
                act.day = activity.limit;
                if(act.qtdDone != act.limit) info.push(act);
            } 
            info.sort((a,b)=> {return a.day-b.day;})
            res.ok(info)
        }else{
            return res.status(500).send({
                success: false,
                message: "Dados não encontrados."
            });
        }
    },

    getAllActivitiesByTeamManager: async function (req, res) {
        var time = req.param('time');
        let months = [];
        let quarters = [[1,2,3],[4,5,6],[7,8,9],[10,11,12]];
        let thisMonth= new Date().toLocaleDateString("pt-BR").split("-")[1];
        let thisYear= new Date().toLocaleDateString("pt-BR").split("-")[0];
        

        if(time == 'monthly'){
           
            let registers = await Activityuser.find({idUserParent: req.user.id});
            let salesmen =  await User.find({idParent: req.user.id}).populate('idTeam');
            if(registers){
                for(let s of salesmen){
                    s.activities = [];
                    s.totalPoints = 0;
                    let registersByActivity = await Activityuser.find({idUser: s.id});
                    for(let r of registersByActivity){
                        r.month = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                        r.year = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                        if(r.month == thisMonth && r.year == thisYear){
                            s.activities.push(r);
                            s.totalPoints += r.score;
                        }
                    }
                    s.qtdActivities = s.activities.length;
                }   
                salesmen.sort((a,b)=> {return b.totalPoints-a.totalPoints;})
                return res.ok(salesmen);
            }else{
                return res.ok();
            }
        }
        if(time == 'quarterly'){
            let activities = await Activity.find();

            for(let q of quarters){
                let findQuarter = q.find(element => {
                 if(element == thisMonth) return true;
                });
                if(findQuarter) months = q;
             }
     
             let salesmen = await User.find({idParent:req.user.id}).populate('idTeam');
             if(salesmen){
                 for(let user of salesmen){
                     user.totalPoints = 0;
                     user.activities = [];
                     for(let activity of activities){
                         let registersByActivity = await Activityuser.find({idUser: user.id, idActivity: activity.id}).populate('idUser').populate('idUserParent').populate('idActivity');
                         for(let m of months){
                             for(let r of registersByActivity){
                                 r.month = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                                 r.year = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                                 if(r.month == m && r.year == thisYear){
                                    user.activities.push(r);
                                    user.totalPoints += r.score;
                                 }
                             }
                         }  
                     } 
                     user.qtdActivities = user.activities.length;
                 }
                 salesmen.sort((a,b)=> {return b.totalPoints-a.totalPoints;})
                 res.ok(salesmen);
             }else{
                 return res.status(500).send({
                     success: false,
                     message: "Dados não encontrados."
                 });
             }
        }
        if(time == 'annually'){
            let activities = await Activity.find();
             let salesmen = await User.find({idParent:req.user.id}).populate('idTeam');
             if(salesmen){
                 for(let user of salesmen){
                     user.totalPoints = 0;
                     user.activities = [];
                     for(let activity of activities){
                         let registersByActivity = await Activityuser.find({idUser: user.id, idActivity: activity.id}).populate('idUser').populate('idUserParent').populate('idActivity');
                         for(let i = 12; i >= 1; i--){
                             for(let r of registersByActivity){
                                 r.month = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                                 r.year = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                                 if(r.month == i && r.year == thisYear){
                                    user.activities.push(r);
                                    user.totalPoints += r.score;
                                 }
                             }
                         }  
                     } 
                     user.qtdActivities = user.activities.length;
                 }
                 salesmen.sort((a,b)=> {return b.totalPoints-a.totalPoints;})
                 res.ok(salesmen);
             }else{
                 return res.status(500).send({
                     success: false,
                     message: "Dados não encontrados."
                 });
             }
        }
    },

    getAllActivitiesByRegionalManager: async function (req, res) {
        var time = req.param('time');
        let activities = await Activity.find();
        let months = [];
        let quarters = [[1,2,3],[4,5,6],[7,8,9],[10,11,12]];
        let thisMonth= new Date().toLocaleDateString("pt-BR").split("-")[1];
        let thisYear= new Date().toLocaleDateString("pt-BR").split("-")[0];

        if(time == 'monthly'){
            let teamManagers =  await User.find({idParent: req.user.id}).populate('idTeam');
            if(teamManagers){
                for(let tm of teamManagers){
                    tm.qtdActivities = 0;
                    tm.totalPoints = 0;
                    tm.subordinates = [];
                    let usersByRegionalManager = await User.find({idParent:tm.id}).populate('idRole').populate('idTeam').populate('idParent');
                    if(usersByRegionalManager){
                        for(let user of usersByRegionalManager){
                            user.activities = [];
                            for(let activity of activities){
                                let registersByActivity = await Activityuser.find({idUser: user.id, idActivity: activity.id}).populate('idUser').populate('idUserParent').populate('idActivity');
                                let filteredRegisters = registersByActivity.filter(element=>{
                                    element.month = new Date(element.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                                    element.year = new Date(element.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                                    return element.month == thisMonth && element.year == thisYear;
                                });
                                user.activities.push(filteredRegisters);
                                tm.qtdActivities += filteredRegisters.length;
                                for(let f of filteredRegisters){
                                    tm.totalPoints += f.score;
                                }
                            } 
                            user.activities = [].concat.apply([],user.activities);
                            tm.subordinates.push(user);
                        }
                    }
                }
                res.ok(teamManagers);
            }else{
                return res.status(500).send({
                    success: false,
                    message: "Dados não encontrados."
                });
            }
        }
        if(time == 'quarterly'){
            for(let q of quarters){
                let findQuarter = q.find(element => {
                 if(element == thisMonth) return true;
                });
                if(findQuarter) months = q;
             }
            let teamManagers =  await User.find({idParent: req.user.id}).populate('idTeam');
            if(teamManagers){
                for(let tm of teamManagers){
                    tm.qtdActivities = 0;
                    tm.totalPoints = 0;
                    tm.subordinates = [];
                    let usersByRegionalManager = await User.find({idParent:tm.id}).populate('idRole').populate('idTeam').populate('idParent');
                    if(usersByRegionalManager){
                        for(let user of usersByRegionalManager){
                            user.activities = [];
                            for(let activity of activities){
                                let registersByActivity = await Activityuser.find({idUser: user.id, idActivity: activity.id}).populate('idUser').populate('idUserParent').populate('idActivity');
                                for(let m of months){
                                    let filteredRegisters = registersByActivity.filter(element=>{
                                        element.month = new Date(element.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                                        element.year = new Date(element.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                                        return element.month == m && element.year == thisYear;
                                    });
                                    user.activities.push(filteredRegisters);
                                    tm.qtdActivities += filteredRegisters.length;
                                    for(let f of filteredRegisters){
                                        tm.totalPoints += f.score;
                                    }
                                }
                            } 
                            user.activities = [].concat.apply([],user.activities);
                            tm.subordinates.push(user);
                        }
                    }
                }
                res.ok(teamManagers);
            }else{
                return res.status(500).send({
                    success: false,
                    message: "Dados não encontrados."
                });
            }
        }
        if(time == 'annually'){
            let teamManagers =  await User.find({idParent: req.user.id}).populate('idTeam');
            if(teamManagers){
                for(let tm of teamManagers){
                    tm.qtdActivities = 0;
                    tm.totalPoints = 0;
                    tm.subordinates = [];
                    let usersByRegionalManager = await User.find({idParent:tm.id}).populate('idRole').populate('idTeam').populate('idParent');
                    if(usersByRegionalManager){
                        for(let user of usersByRegionalManager){
                            user.activities = [];
                            for(let activity of activities){
                                let registersByActivity = await Activityuser.find({idUser: user.id, idActivity: activity.id}).populate('idUser').populate('idUserParent').populate('idActivity');
                                for(let i = 12; i >= 1; i--){
                                    let filteredRegisters = registersByActivity.filter(element=>{
                                        element.month = new Date(element.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                                        element.year = new Date(element.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                                        return element.month == i && element.year == thisYear;
                                    });
                                    user.activities.push(filteredRegisters);
                                    tm.qtdActivities += filteredRegisters.length;
                                    for(let f of filteredRegisters){
                                        tm.totalPoints += f.score;
                                    }
                                }
                            } 
                            user.activities = [].concat.apply([],user.activities);
                            tm.subordinates.push(user);
                        }
                    }
                }
                res.ok(teamManagers);
            }else{
                return res.status(500).send({
                    success: false,
                    message: "Dados não encontrados."
                });
            }
        }

        
    },

    getAllActivitiesByTeam: async function (req, res) {
        var time = req.param('time');
        let teams = await Team.find();
        let activities = await Activity.find();
        let months = [];
        let quarters = [[1,2,3],[4,5,6],[7,8,9],[10,11,12]];
        let thisMonth= new Date().toLocaleDateString("pt-BR").split("-")[1];
        let thisYear= new Date().toLocaleDateString("pt-BR").split("-")[0];
        
        if(time == 'monthly'){
            if(teams){
                for(let t of teams){
                    let aux = await User.find({idTeam:t.id}).populate('idRole').populate('idTeam').populate('idParent');
                    let usersByTeam = aux.filter(element=> {
                        return element.idRole.isSalesman == 1;
                    });
                    t.members = usersByTeam;
                    t.qtdActivities = 0;
                    t.totalPoints = 0;
                    if(usersByTeam){
                        for(let user of usersByTeam){
                            user.qtdActivities = 0;
                            for(let activity of activities){
                                let registersByActivity = await Activityuser.find({idUser: user.id, idActivity: activity.id}).populate('idUser').populate('idUserParent').populate('idActivity');
                                let filteredRegisters = registersByActivity.filter(element=>{
                                    element.month = new Date(element.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                                    element.year = new Date(element.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                                    return (element.month == thisMonth && element.year == thisYear);
                                });
                                for(let f of filteredRegisters){
                                    t.totalPoints += f.score;
                                }
                                user.qtdActivities += filteredRegisters.length;
                            } 
                            t.qtdActivities += user.qtdActivities;
                        }
                    }
                }
                teams.sort((a,b)=> {return b.totalPoints-a.totalPoints;})
                res.ok(teams)
            }else{
                return res.status(500).send({
                    success: false,
                    message: "Dados não encontrados."
                });
            }
        }
        
        if(time == 'quarterly'){
            for(let q of quarters){
                let findQuarter = q.find(element => {
                 if(element == thisMonth) return true;
                });
                if(findQuarter) months = q;
             }
            if(teams){
                for(let t of teams){
                    let aux = await User.find({idTeam:t.id}).populate('idRole').populate('idTeam').populate('idParent');
                    let usersByTeam = aux.filter(element=> {
                        return element.idRole.isSalesman == 1;
                    });
                    t.members = usersByTeam;
                    t.qtdActivities = 0;
                    t.totalPoints = 0;
                    if(usersByTeam){
                        for(let user of usersByTeam){
                            user.qtdActivities = 0;
                            for(let activity of activities){
                                let registersByActivity = await Activityuser.find({idUser: user.id, idActivity: activity.id}).populate('idUser').populate('idUserParent').populate('idActivity');
                                for(let m of months){
                                    let filteredRegisters = registersByActivity.filter(element=>{
                                        element.month = new Date(element.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                                        element.year = new Date(element.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                                        return (element.month == m && element.year == thisYear);
                                    });
                                    for(let f of filteredRegisters){
                                        t.totalPoints += f.score;
                                    }
                                    user.qtdActivities += filteredRegisters.length;
                                }
                            } 
                            t.qtdActivities += user.qtdActivities;
                        }
                    }
                }
                teams.sort((a,b)=> {return b.totalPoints-a.totalPoints;})
                res.ok(teams)
            }else{
                return res.status(500).send({
                    success: false,
                    message: "Dados não encontrados."
                });
            }
        }
        if(time == 'annually'){
            if(teams){
                for(let t of teams){
                    let aux = await User.find({idTeam:t.id}).populate('idRole').populate('idTeam').populate('idParent');
                    let usersByTeam = aux.filter(element=> {
                        return element.idRole.isSalesman == 1;
                    });
                    t.members = usersByTeam;
                    t.qtdActivities = 0;
                    t.totalPoints = 0;
                    if(usersByTeam){
                        for(let user of usersByTeam){
                            user.qtdActivities = 0;
                            for(let activity of activities){
                                let registersByActivity = await Activityuser.find({idUser: user.id, idActivity: activity.id}).populate('idUser').populate('idUserParent').populate('idActivity');
                                for(let i = 12; i >= 1; i--){
                                    let filteredRegisters = registersByActivity.filter(element=>{
                                        element.month = new Date(element.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                                        element.year = new Date(element.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                                        return (element.month == i && element.year == thisYear);
                                    });
                                    for(let f of filteredRegisters){
                                        t.totalPoints += f.score;
                                    }
                                    user.qtdActivities += filteredRegisters.length;
                                }
                            } 
                            t.qtdActivities += user.qtdActivities;
                        }
                    }
                }
                teams.sort((a,b)=> {return b.totalPoints-a.totalPoints;})
                res.ok(teams)
            }else{
                return res.status(500).send({
                    success: false,
                    message: "Dados não encontrados."
                });
            }
        }

    },


    
    getActivityById: async function (req, res) {
        var id = req.param('id');
      
        //Não foi enviado parâmetro
		if(!id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }

        let data = await Activity.find({ id: id });
        return res.ok(data);
       
    },



    createActivity: async function (req, res) {
        let params = req.allParams();
		
		//Não foi enviado parâmetro
		if(Object.keys(params).length == 0){
			return res.status(500).send({
                    success: false,
                    message: "Não foi enviado parâmetros"
                });
		}
		
		let data = await Activity.create(params).fetch();

        if (data) {
            return res.ok({
                success: true,
            });
        } else {
            return res.status(500).send({
                success: false,
                message: "Ocorreu um erro durante o cadastro"
            });
        }
    },


    
    updateActivity: async function (req, res) {

        let params = req.allParams();

        //Não foi enviado parâmetro
		if(Object.keys(params).length == 0){
			return res.status(500).send({
                    success: false,
                    message: "Não foi enviado parâmetros"
                });
		}else if(!params.id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }

        var data = await Activity.updateOne({ "id": params.id }).set(params);

        if (data) {
            return res.ok({
                success: true,
            });
        } else {
            return res.status(500).send({
                success: false,
                message: "ID informado, não foi encontrado"
            });
        }
    },


    
    deleteActivityById: async function (req, res) {
        var id = req.param('id');
        //Não foi enviado parâmetro
		if(!id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }

        var data = await Activity.destroyOne({
            id: id,
        }, function (err, success) {

            if (err) {             
                return res.status(500).send({
                    success: false,
                    message: "Não foi possível deletar",
                    err: err
                });
            } else {
                return res.ok({
                            success: true,
                        });
            }
        });
    }};
