/**
* Activityuser.js
*
* @description :: activityuser controller imported from localhost MySql server at 18/3/2020 16:24:20.
* @docs        :: http://sailsjs.org/#!documentation/controllers
* @routes      :: /* Activityuser /*
*              'get /activityuser/getAllActivityuser/': 'ActivityuserController.getAllActivityuser',
*              'get /activityuser/getActivityuserById/:id/': 'ActivityuserController.getActivityuserById',
*              'post /activityuser/createActivityuser/': 'ActivityuserController.createActivityuser', 
*              'put /activityuser/updateActivityuser/': 'ActivityuserController.updateActivityuser', 
*              'detele /activityuser/deleteActivityuserById/:id/': 'ActivityuserController.deleteActivityuserById',
* 
*/


module.exports = {

    
    getAllActivityuser: async function (req, res) {
        let data = await Activityuser.find().populate('idUser').populate('idUserParent').populate('idActivity').sort('id DESC');
        let user = await User.findOne({id: req.user.id});
        if(user){
            let type = await Role.findOne({id: user.idRole});
            if(type.isAdmin == 1){
                res.ok(data);
            }
            else if(type.isTeamManager == 1){
                let activityUserByTeamManagerId = data.filter(element =>{
                    return element.idUser.idParent == req.user.id;
                })
                res.ok(activityUserByTeamManagerId);
            }
        }else{
            return res.status(500).send({
                success: false,
                message: "Registros não encontrados."
            });
        }
    },

    
    getActivityuserById: async function (req, res) {
        var id = req.param('id');
      
        //Não foi enviado parâmetro
		if(!id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }

        let data = await Activityuser.find({ id: id });
        return res.ok(data);
       
    },


    getActivityUserByUserId: async function (req, res) {
        var id = req.param('id');
        //Não foi enviado parâmetro
		if(!id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }

        let data = await Activityuser.find({ idUser: id }).populate('idUser').populate('idActivity').populate('idUserParent').sort('id DESC');
        return res.ok(data);
       
    },


    createActivityuser: async function (req, res) {
        let params = req.allParams();

        let today = new Date().toLocaleDateString("pt-BR").split("-")[2];
        let thisMonth = new Date().toLocaleDateString("pt-BR").split("-")[1];
        
        let activity = await Activity.findOne({ id: params.idActivity });

        if(today < activity.startLimit || today > activity.limit){
            return res.ok({
                success: false,
                message: "Período de lançamento incorreto."
            });
        }else{
            let registers = await Activityuser.find({ idUser: params.idUser, idActivity: params.idActivity }).populate('idUser').populate('idActivity').populate('idUserParent').sort('id DESC');

            if(registers.length > 0){
                for(let r of registers){
                    r.day = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[2];
                    r.month = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                }
                let filteredRegistersByDay = registers.filter(element =>{
                    return element.day == today;
                });
                let filteredRegistersByMonth = registers.filter(element =>{
                    return element.month == thisMonth;
                });

                if(filteredRegistersByDay.length >= activity.dailyLimit){
                    return res.ok({
                        success: false,
                        message: "Limite diário atingido."
                    });
                }

                if(filteredRegistersByMonth.length >= activity.monthLimit){
                    return res.ok({
                        success: false,
                        message: "Meta mensal já atingida."
                    });
                }
            }

            let data = await Activityuser.create(params).fetch();

            if (data) {
                return res.ok({
                    success: true,
                });
            } else {
                return res.status(500).send({
                    success: false,
                    message: "Ocorreu um erro durante o cadastro"
                });
            }
        }
    },


    
    updateActivityuser: async function (req, res) {

        let params = req.allParams();

        //Não foi enviado parâmetro
		if(Object.keys(params).length == 0){
			return res.status(500).send({
                    success: false,
                    message: "Não foi enviado parâmetros"
                });
		}else if(!params.id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }

        const activityUser = await Activityuser.findOne({"id": params.id});
        if(activityUser){
            if(params.idActivity != activityUser.idActivity){
                return res.ok({
                    success: false,
                    message: "Não é possível alterar a atividade"
                });
            }
        }

        var data = await Activityuser.updateOne({ "id": params.id }).set(params);

        if (data) {
            return res.ok({
                success: true,
            });
        } else {
            return res.status(500).send({
                success: false,
                message: "ID informado, não foi encontrado"
            });
        }
    },


    
    deleteActivityUserById: async function (req, res) {
        var id = req.param('id');
        //Não foi enviado parâmetro
		if(!id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }

        var data = await Activityuser.destroyOne({
            id: id,
        }, function (err, success) {

            if (err) {             
                return res.status(500).send({
                    success: false,
                    message: "Não foi possível deletar",
                    err: err
                });
            } else {
                return res.ok({
                    success: true,
                });
            }
        });
    }};
