/**
* Role.js
*
* @description :: role controller imported from localhost MySql server at 18/3/2020 16:24:20.
* @docs        :: http://sailsjs.org/#!documentation/controllers
* @routes      :: /* Role /*
*              'get /role/getAllRole/': 'RoleController.getAllRole',
*              'get /role/getRoleById/:id/': 'RoleController.getRoleById',
*              'post /role/createRole/': 'RoleController.createRole', 
*              'put /role/updateRole/': 'RoleController.updateRole', 
*              'detele /role/deleteRoleById/:id/': 'RoleController.deleteRoleById',
* 
*/


module.exports = {

    
    getAllRole: async function (req, res) {
            let data = await Role.find();
            return res.ok(data);
    },


    
    getRoleById: async function (req, res) {
        var id = req.param('id');
      
        //Não foi enviado parâmetro
		if(!id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }

        let data = await Role.find({ id: id });
        return res.ok(data);
       
    },



    createRole: async function (req, res) {
        let params = req.allParams();
		
		//Não foi enviado parâmetro
		if(Object.keys(params).length == 0){
			return res.status(500).send({
                    success: false,
                    message: "Não foi enviado parâmetros"
                });
		}
		
		let data = await Role.create(params).fetch();

        if (data) {
            return res.ok({
                success: true,
            });
        } else {
            return res.status(500).send({
                success: false,
                message: "Ocorreu um erro durante o cadastro"
            });
        }
    },


    
    updateRole: async function (req, res) {

        let params = req.allParams();

        //Não foi enviado parâmetro
		if(Object.keys(params).length == 0){
			return res.status(500).send({
                    success: false,
                    message: "Não foi enviado parâmetros"
                });
		}else if(!params.id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }

        var data = await Role.updateOne({ "id": params.id }).set(params);

        if (data) {
            return res.ok({
                success: true,
            });
        } else {
            return res.status(500).send({
                success: false,
                message: "ID informado, não foi encontrado"
            });
        }
    },


    
    deleteRoleById: async function (req, res) {
        var id = req.param('id');

        //Não foi enviado parâmetro
		if(!id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }

        var data = await Role.destroyOne({
            id: id,
        }, function (err, success) {

            if (err) {             
                return res.status(500).send({
                    success: false,
                    message: "Não foi possível deletar",
                    err: err
                });
            } else {
                return res.ok({
                            success: true,
                        });
            }
        });
    }};
