/**
* Sale.js
*
* @description :: sale controller imported from localhost MySql server at 18/3/2020 16:24:20.
* @docs        :: http://sailsjs.org/#!documentation/controllers
* @routes      :: /* Sale /*
*              'get /sale/getAllSale/': 'SaleController.getAllSale',
*              'get /sale/getSaleById/:id/': 'SaleController.getSaleById',
*              'post /sale/createSale/': 'SaleController.createSale', 
*              'put /sale/updateSale/': 'SaleController.updateSale', 
*              'detele /sale/deleteSaleById/:id/': 'SaleController.deleteSaleById',
* 
*/


module.exports = {

    
    getAllSales: async function (req, res) {
            let data = await Sale.find().sort('id DESC');
            return res.ok(data);
    },

    getAllSalesByTeamManager: async function (req, res) {
        var time = req.param('time');
        let months = [];
        let thisMonth= new Date().toLocaleDateString("pt-BR").split("-")[1];
        let thisYear= new Date().toLocaleDateString("pt-BR").split("-")[0];
        let quarters = [[1,2,3],[4,5,6],[7,8,9],[10,11,12]];

        if(time == 'monthly'){
            let salesmen =  await User.find({idParent: req.user.id}).populate('idTeam');
            if(salesmen){
                for(let s of salesmen){
                    s.sales = [];
                    s.totalAmmount = 0;
                    let registersBySale = await Saleuser.find({idUser: s.id, status: 1}).populate('idUser');
                    for(let r of registersBySale){
                        r.month = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                        r.year = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                        if(r.month == thisMonth && r.year == thisYear){
                            s.sales.push(r);
                            s.totalAmmount += r.price;
                        }
                    }
                    s.qtdSales = s.sales.length;
                }
                salesmen.sort((a,b)=> {return b.totalAmmount-a.totalAmmount;})
                return res.ok(salesmen);
            }else{
                return res.status(500).send({
                    success: false,
                    message: "Dados não encontrados."
                });
            }
        }
        if(time == 'quarterly'){
            let sales = await Sale.find();

            for(let q of quarters){
                let findQuarter = q.find(element => {
                 if(element == thisMonth) return true;
                });
                if(findQuarter) months = q;
             }
     
             let salesmen = await User.find({idParent:req.user.id}).populate('idRole').populate('idTeam').populate('idParent');
             if(salesmen){
                 for(let user of salesmen){
                    user.totalAmmount = 0;
                    user.sales = [];
                     for(let sale of sales){
                         let registersBySale = await Saleuser.find({idUser: user.id, idSale: sale.id}).populate('idUser').populate('idUserParent').populate('idSale');
                         for(let m of months){
                             for(let r of registersBySale){
                                 r.month = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                                 r.year = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                                 if (r.month == m && r.year == thisYear && r.status == 1) {
                                     user.sales.push(r);
                                     user.totalAmmount += r.price;
                                 }
                             }
                         }  
                     } 
                     user.qtdSales = user.sales.length;
                 }
                 salesmen.sort((a,b)=> {return b.totalAmmount-a.totalAmmount;})
                 res.ok(salesmen);
             }else{
                 return res.status(500).send({
                     success: false,
                     message: "Dados não encontrados."
                 });
             }
        }
        if(time == 'annually'){
            let sales = await Sale.find();
     
             let salesmen = await User.find({idParent:req.user.id}).populate('idRole').populate('idTeam').populate('idParent');
             if(salesmen){
                 for(let user of salesmen){
                    user.totalAmmount = 0;
                    user.sales = [];
                     for(let sale of sales){
                         let registersBySale = await Saleuser.find({idUser: user.id, idSale: sale.id}).populate('idUser').populate('idUserParent').populate('idSale');
                         for(let i = 12; i >= 1; i--){
                             for(let r of registersBySale){
                                 r.month = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                                 r.year = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                                 if (r.month == i && r.year == thisYear && r.status == 1) {
                                     user.sales.push(r);
                                     user.totalAmmount += r.price;
                                 }
                             }
                         }  
                     } 
                     user.qtdSales = user.sales.length;
                 }
                 salesmen.sort((a,b)=> {return b.totalAmmount-a.totalAmmount;})
                 res.ok(salesmen);
             }else{
                 return res.status(500).send({
                     success: false,
                     message: "Dados não encontrados."
                 });
             }
        }
    },

    getAllSalesByRegionalManager: async function (req, res) {
        var time = req.param('time');
        let months = [];
        let quarters = [[1,2,3],[4,5,6],[7,8,9],[10,11,12]];
        let thisMonth= new Date().toLocaleDateString("pt-BR").split("-")[1];
        let thisYear= new Date().toLocaleDateString("pt-BR").split("-")[0];
        
        if(time == 'monthly'){
            let teamManagers =  await User.find({idParent: req.user.id}).populate('idTeam');
            if(teamManagers){
                for(let tm of teamManagers){
                    tm.sales = [];
                    tm.totalAmmount = 0;
                    let registersBySale = await Saleuser.find({idUserParent: tm.id, status: 1}).populate('idUser').populate('idUserParent').populate('idSale');
                    for(let r of registersBySale){
                        r.month = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                        r.year = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                        if(r.month == thisMonth && r.year == thisYear){
                            tm.sales.push(r);
                            tm.totalAmmount += r.price;
                        }
                    }
                    tm.sales = [].concat.apply([],tm.sales);
                    tm.qtdSales = tm.sales.length;
                }
                teamManagers.sort((a,b)=> {return b.totalAmmount-a.totalAmmount;})
                return res.ok(teamManagers);
            }else{
                return res.status(500).send({
                    success: false,
                    message: "Dados não encontrados."
                });
            }
        }

        if(time == 'quarterly'){
            for(let q of quarters){
                let findQuarter = q.find(element => {
                 if(element == thisMonth) return true;
                });
                if(findQuarter) months = q;
             }

            let teamManagers =  await User.find({idParent: req.user.id}).populate('idTeam');
            if(teamManagers){
                for(let tm of teamManagers){
                    tm.sales = [];
                    tm.totalAmmount = 0;
                    let registersBySale = await Saleuser.find({idUserParent: tm.id, status: 1}).populate('idUser').populate('idUserParent').populate('idSale');
                    for(let m of months){
                        for(let r of registersBySale){
                            r.month = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                            r.year = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                            if(r.month == m && r.year == thisYear){
                                tm.sales.push(r);
                                tm.totalAmmount += r.price;
                            }
                        }
                    tm.sales = [].concat.apply([],tm.sales);
                    tm.qtdSales = tm.sales.length;
                    }
                }
                teamManagers.sort((a,b)=> {return b.totalAmmount-a.totalAmmount;})
                return res.ok(teamManagers);
            }else{
                return res.status(500).send({
                    success: false,
                    message: "Dados não encontrados."
                });
            }
        }

        if(time == 'annually'){
            let teamManagers =  await User.find({idParent: req.user.id}).populate('idTeam');
            if(teamManagers){
                for(let tm of teamManagers){
                    tm.sales = [];
                    tm.totalAmmount = 0;
                    let registersBySale = await Saleuser.find({idUserParent: tm.id, status: 1}).populate('idUser').populate('idUserParent').populate('idSale');
                    for(let i = 12; i >= 1; i--){
                        for(let r of registersBySale){
                            r.month = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                            r.year = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                            if(r.month == i && r.year == thisYear){
                                tm.sales.push(r);
                                tm.totalAmmount += r.price;
                            }
                        }
                    tm.sales = [].concat.apply([],tm.sales);
                    tm.qtdSales = tm.sales.length;
                    }
                }
                teamManagers.sort((a,b)=> {return b.totalAmmount-a.totalAmmount;})
                return res.ok(teamManagers);
            }else{
                return res.status(500).send({
                    success: false,
                    message: "Dados não encontrados."
                });
            }
               
        }
    },

    getAllSalesByTeam: async function (req, res) {
        var time = req.param('time');
        let teams = await Team.find();
        let sales = await Sale.find();
        let months = [];
        let quarters = [[1,2,3],[4,5,6],[7,8,9],[10,11,12]];
        let thisMonth= new Date().toLocaleDateString("pt-BR").split("-")[1];
        let thisYear= new Date().toLocaleDateString("pt-BR").split("-")[0];

        if(time == 'monthly'){
            if(teams){
                for(let t of teams){
                    let aux = await User.find({idTeam:t.id}).populate('idRole').populate('idTeam').populate('idParent');
                    let usersByTeam = aux.filter(element=> {
                        return element.idRole.isSalesman == 1;
                    });
                    t.members = usersByTeam;
                    t.qtdSales = 0;
                    t.totalAmmount = 0;
                    if(usersByTeam){
                        for(let user of usersByTeam){
                            user.qtdSale = 0;
                            for(let sale of sales){
                                let registersBySale = await Saleuser.find({idUser: user.id, idSale: sale.id}).populate('idUser').populate('idUserParent').populate('idSale');
                                let filteredRegisters = registersBySale.filter(element=>{
                                    element.month = new Date(element.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                                    element.year = new Date(element.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                                    return (element.month == thisMonth && element.year == thisYear && element.status == 1);
                                });
                                for(let f of filteredRegisters){
                                    t.totalAmmount += f.price;
                                }
                                user.qtdSale += filteredRegisters.length;
                            } 
                            t.qtdSales += user.qtdSale;
                        }
                    }
                }
                teams.sort((a,b)=> {return b.totalAmmount-a.totalAmmount;})
                res.ok(teams)
            }else{
                return res.status(500).send({
                    success: false,
                    message: "Dados não encontrados."
                });
            }
        }
        if(time == 'quarterly'){
            for(let q of quarters){
                let findQuarter = q.find(element => {
                 if(element == thisMonth) return true;
                });
                if(findQuarter) months = q;
             }
            if(teams){
                for(let t of teams){
                    let aux = await User.find({idTeam:t.id}).populate('idRole').populate('idTeam').populate('idParent');
                    let usersByTeam = aux.filter(element=> {
                        return element.idRole.isSalesman == 1;
                    });
                    t.members = usersByTeam;
                    t.qtdSales = 0;
                    t.totalAmmount = 0;
                    if(usersByTeam){
                        for(let user of usersByTeam){
                            user.qtdSale = 0;
                            for(let sale of sales){
                                let registersBySale = await Saleuser.find({idUser: user.id, idSale: sale.id}).populate('idUser').populate('idUserParent').populate('idSale');
                                for(let m of months){
                                let filteredRegisters = registersBySale.filter(element=>{
                                        element.month = new Date(element.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                                        element.year = new Date(element.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                                        return (element.month == m && element.year == thisYear && element.status == 1);
                                    });
                                    for(let f of filteredRegisters){
                                        t.totalAmmount += f.price;
                                    }
                                    user.qtdSale += filteredRegisters.length;
                                }
                            } 
                            t.qtdSales += user.qtdSale;
                        }
                    }
                }
                teams.sort((a,b)=> {return b.totalAmmount-a.totalAmmount;})
                res.ok(teams)
            }else{
                return res.status(500).send({
                    success: false,
                    message: "Dados não encontrados."
                });
            }
        }
        if(time == 'annually'){
            if(teams){
                for(let t of teams){
                    let aux = await User.find({idTeam:t.id}).populate('idRole').populate('idTeam').populate('idParent');
                    let usersByTeam = aux.filter(element=> {
                        return element.idRole.isSalesman == 1;
                    });
                    t.members = usersByTeam;
                    t.qtdSales = 0;
                    t.totalAmmount = 0;
                    if(usersByTeam){
                        for(let user of usersByTeam){
                            user.qtdSale = 0;
                            for(let sale of sales){
                                let registersBySale = await Saleuser.find({idUser: user.id, idSale: sale.id}).populate('idUser').populate('idUserParent').populate('idSale');
                                for(let i = 12; i >= 1; i--){
                                let filteredRegisters = registersBySale.filter(element=>{
                                        element.month = new Date(element.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                                        element.year = new Date(element.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                                        return (element.month == i && element.year == thisYear && element.status == 1);
                                    });
                                    for(let f of filteredRegisters){
                                        t.totalAmmount += f.price;
                                    }
                                    user.qtdSale += filteredRegisters.length;
                                }
                            } 
                            t.qtdSales += user.qtdSale;
                        }
                    }
                }
                teams.sort((a,b)=> {return b.totalAmmount-a.totalAmmount;})
                res.ok(teams)
            }else{
                return res.status(500).send({
                    success: false,
                    message: "Dados não encontrados."
                });
            }
        }

        
    },


    
    getSaleById: async function (req, res) {
        var id = req.param('id');
      
        //Não foi enviado parâmetro
		if(!id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }

        let data = await Sale.find({ id: id });
        return res.ok(data);
       
    },



    createSale: async function (req, res) {
        let params = req.allParams();
		
		//Não foi enviado parâmetro
		if(Object.keys(params).length == 0){
			return res.status(500).send({
                    success: false,
                    message: "Não foi enviado parâmetros"
                });
		}
		
		let data = await Sale.create(params).fetch();

        if (data) {
            return res.ok({
                success: true,
            });
        } else {
            return res.status(500).send({
                success: false,
                message: "Ocorreu um erro durante o cadastro"
            });
        }
    },


    
    updateSale: async function (req, res) {

        let params = req.allParams();

        //Não foi enviado parâmetro
		if(Object.keys(params).length == 0){
			return res.status(500).send({
                    success: false,
                    message: "Não foi enviado parâmetros"
                });
		}else if(!params.id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }

        var data = await Sale.updateOne({ "id": params.id }).set(params);

        if (data) {
            return res.ok({
                success: true,
            });
        } else {
            return res.status(500).send({
                success: false,
                message: "ID informado, não foi encontrado"
            });
        }
    },

    


    
    deleteSaleById: async function (req, res) {
        var id = req.param('id');
        //Não foi enviado parâmetro
		if(!id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }

        var data = await Sale.destroyOne({
            id: id,
        }, function (err, success) {
            if (err) {             
                return res.status(500).send({
                    success: false,
                    message: "Não foi possível deletar",
                    err: err
                });
            } else {
                return res.ok({
                    success: true,
                });
            }
        });
    }};
