/**
* Saleuser.js
*
* @description :: saleuser controller imported from localhost MySql server at 18/3/2020 16:24:20.
* @docs        :: http://sailsjs.org/#!documentation/controllers
* @routes      :: /* Saleuser /*
*              'get /saleuser/getAllSaleuser/': 'SaleuserController.getAllSaleuser',
*              'get /saleuser/getSaleuserById/:id/': 'SaleuserController.getSaleuserById',
*              'post /saleuser/createSaleuser/': 'SaleuserController.createSaleuser', 
*              'put /saleuser/updateSaleuser/': 'SaleuserController.updateSaleuser', 
*              'detele /saleuser/deleteSaleuserById/:id/': 'SaleuserController.deleteSaleuserById',
* 
*/


module.exports = {

    
    getAllSaleuser: async function (req, res) {
        let data = await Saleuser.find().populate('idUser').populate('idUserParent').populate('idSale').sort('id DESC');
        let user = await User.findOne({id: req.user.id});
        if(user){
            let type = await Role.findOne({id: user.idRole});
            if(type.isAdmin == 1 || type.isGCD == 1){
                res.ok(data);
            }
            else if(type.isTeamManager == 1){
                let saleUserByTeamManagerId = data.filter(element =>{
                    return element.idUser.idParent == req.user.id;
                })
                res.ok(saleUserByTeamManagerId);
            }
        }else{
            return res.status(500).send({
                success: false,
                message: "Registros não encontrados."
            });
        }
    },

    getOpenSaleuser: async function (req, res) {
        let data = await Saleuser.find({status:2}).populate('idUser').populate('idUserParent').populate('idSale').sort('id DESC');
        if(data){
           res.ok(data);
        }else{
            return res.status(500).send({
                success: false,
                message: "Registros não encontrados."
            });
        }
    },

    getSalesmanSales: async function(req,res){
        let thisMonth= new Date().toLocaleDateString("pt-BR").split("-")[1];
        let thisYear= new Date().toLocaleDateString("pt-BR").split("-")[0];
        let salesman = await User.findOne({id:req.user.id}).populate('idTeam');
        if(salesman){
            salesDone = [];
            let sales = await Saleuser.find({idUser: salesman.id}).populate('idUser').populate('idUserParent').populate('idSale'); 
            for(let s of sales){
                s.month = new Date(s.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                s.year = new Date(s.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                if(s.month == thisMonth && s.year == thisYear){
                    salesDone.push(s);
                }              
            }
            res.ok(salesDone)
        }else{
            return res.status(500).send({
                success: false,
                message: "Dados não encontrados."
            });
        }
    },


    
    getSaleuserById: async function (req, res) {
        var id = req.param('id');
      
        //Não foi enviado parâmetro
		if(!id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }

        let data = await Saleuser.find({ id: id });
        return res.ok(data);
       
    },



    createSaleuser: async function (req, res) {
        let params = req.allParams();
		
		//Não foi enviado parâmetro
		if(Object.keys(params).length == 0){
			return res.status(500).send({
                    success: false,
                    message: "Não foi enviado parâmetros"
                });
        }

        let register = await Saleuser.find({idSale: params.idSale,block:params.block,apartment:params.apartment}).populate('idUserParent').populate('idSale');
        if(register){
            for(let r of register){
                if(r.status != 0){
                    return res.send({
                        success: false,
                        register : r
                    });
                }
            }
        }

        let data = await Saleuser.create(params).fetch();
        if(data){
            res.ok({
                success:true
            })
        }
        else{
            return res.status(500).send({
                success: false,
                message: "Ocorreu um erro durante o lançamento!"
            });
        }
        
    },


    
    updateSaleuser: async function (req, res) {

        let params = req.allParams();

        //Não foi enviado parâmetro
		if(Object.keys(params).length == 0){
			return res.status(500).send({
                    success: false,
                    message: "Não foi enviado parâmetros"
                });
		}else if(!params.id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }
        let register = await Saleuser.findOne({id: params.id}).populate('idUserParent').populate('idSale');
        let hasRegister = await Saleuser.find({idSale: params.idSale,block:params.block,apartment:params.apartment}).populate('idUserParent').populate('idSale');

        if(hasRegister){
            for(let register of hasRegister){
                if(register.status == 0) continue;
                if(register.id != params.id){
                    return res.ok({
                        success: false,
                        register: hasRegister[0]
                    });
                }
            }

        }

        if(register){
            var data = await Saleuser.updateOne({ "id": params.id }).set(params);
                if(data){
                    res.ok({
                        success:true
                    })
                }else{
                    return res.ok({
                        success: false,
                        message: "Erro ao atualizar o lançamento!"
                    });
                }
            /* if(params.status != 0){
                for(let r of registers){
                    if(r.status != 0 && r.idUserParent.id != req.user.id){
                        return res.ok({
                            success: false,
                            register: r
                        });
                    }
                }
                var data = await Saleuser.updateOne({ "id": params.id }).set(params);
                if(data){
                    res.ok({
                        success:true
                    })
                }else{
                    return res.ok({
                        success: false,
                        message: "Erro ao atualizar o lançamento!"
                    });
                }
            }else{
                var data = await Saleuser.updateOne({ "id": params.id }).set(params);
                if(data){
                    res.ok({
                        success:true
                    })
                }else{
                    return res.ok({
                        success: false,
                        message: "Erro ao atualizar o lançamento!"
                    });
                }
            } */
        }else{
            return res.ok({
                success: false,
                message: "Lançamento não encontrado!"
            });
        }

    },


    
    deleteSaleUserById: async function (req, res) {
        var id = req.param('id');

        //Não foi enviado parâmetro
		if(!id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }

        let saleUser = await Saleuser.find({id:id});
        if(saleUser){
            let sale = await Sale.updateOne({"id": saleUser[0].idSale}).set({isInUse: 0});
            if(sale){
                var data = await Saleuser.destroyOne({
                    id: id,
                }, function (err, success) {
        
                    if (err) {             
                        return res.status(500).send({
                            success: false,
                            message: "Não foi possível deletar",
                            err: err
                        });
                    } else {
                        return res.ok({
                            success: true,
                        });
                    }
        
                });
            }
        }
    }};
