/**
* Sessions.js
*
* @description :: sessions controller imported from localhost MySql server at 18/3/2020 16:24:20.
* @docs        :: http://sailsjs.org/#!documentation/controllers
* @routes      :: /* Sessions /*
*              'get /sessions/getAllSessions/': 'SessionsController.getAllSessions',
*              'get /sessions/getSessionsById/:id/': 'SessionsController.getSessionsById',
*              'post /sessions/createSessions/': 'SessionsController.createSessions', 
*              'put /sessions/updateSessions/': 'SessionsController.updateSessions', 
*              'detele /sessions/deleteSessionsById/:id/': 'SessionsController.deleteSessionsById',
* 
*/


module.exports = {

    
    getAllSessions: async function (req, res) {
            let data = await Sessions.find();
            return res.ok(data);
    },


    
    getSessionsById: async function (req, res) {
        var id = req.param('id');
      
        //Não foi enviado parâmetro
		if(!id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }

        let data = await Sessions.find({ id: id });
        return res.ok(data);
       
    },



    createSessions: async function (req, res) {
        let params = req.allParams();
		
		//Não foi enviado parâmetro
		if(Object.keys(params).length == 0){
			return res.status(500).send({
                    success: false,
                    message: "Não foi enviado parâmetros"
                });
		}
		
		let data = await Sessions.create(params).fetch();

        if (data) {
            return res.ok({
                success: true,
            });
        } else {
            return res.status(500).send({
                success: false,
                message: "Ocorreu um erro durante o cadastro"
            });
        }
    },


    
    updateSessions: async function (req, res) {

        let params = req.allParams();

        //Não foi enviado parâmetro
		if(Object.keys(params).length == 0){
			return res.status(500).send({
                    success: false,
                    message: "Não foi enviado parâmetros"
                });
		}else if(!params.id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }

        var data = await Sessions.updateOne({ "id": params.id }).set(params);

        if (data) {
            return res.ok({
                success: true,
            });
        } else {
            return res.status(500).send({
                success: false,
                message: "ID informado, não foi encontrado"
            });
        }
    },


    
    deleteSessionsById: async function (req, res) {
        var id = req.param('id');

        //Não foi enviado parâmetro
		if(!id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }

        var data = await Sessions.destroyOne({
            id: id,
        }, function (err, success) {

            if (err) {             
                return res.status(500).send({
                    success: false,
                    message: "Não foi possível deletar",
                    err: err
                });
            } else {
                return res.ok({
                            success: true,
                        });
            }
        });
    }};
