/**
* Team.js
*
* @description :: team controller imported from localhost MySql server at 18/3/2020 16:24:20.
* @docs        :: http://sailsjs.org/#!documentation/controllers
* @routes      :: /* Team /*
*              'get /team/getAllTeam/': 'TeamController.getAllTeam',
*              'get /team/getTeamById/:id/': 'TeamController.getTeamById',
*              'post /team/createTeam/': 'TeamController.createTeam', 
*              'put /team/updateTeam/': 'TeamController.updateTeam', 
*              'detele /team/deleteTeamById/:id/': 'TeamController.deleteTeamById',
* 
*/


module.exports = {

    
    getAllTeams: async function (req, res) {
            let data = await Team.find();
            return res.ok(data);
    },

    getSalesmenRanking: async function (req, res) {
        let activities = await Activity.find();
        let thisMonth= new Date().toLocaleDateString("pt-BR").split("-")[1];
        let thisYear= new Date().toLocaleDateString("pt-BR").split("-")[0];
        let totalActivitiesPoints = 0;

        let salesmen = await User.find().populate('idRole');
        if(salesmen){
            salesmen = salesmen.filter(element=>{
                return element.idRole.isSalesman == 1;
            });
            for(let user of salesmen){
                user.activities = [];
                user.points = 0;
                user.numeroPastas = 0;
                user.vgv = 0;
                for(let activity of activities){
                    totalActivitiesPoints += activity.score * activity.monthLimit;
                    let registersByActivity = await Activityuser.find({idUser: user.id, idActivity: activity.id}).populate('idActivity');
                    for(let r of registersByActivity){
                        r.idActivity.idActivityGroup = await Activitygroup.findOne(r.idActivity.idActivityGroup);
                        r.month = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                        r.year = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                        if(r.month == thisMonth && r.year == thisYear){
                            if(r.idActivity.idActivityGroup.name == 'Documentação') user.numeroPastas++;
                            user.activities.push(r);
                            user.points += r.score;
                        }              
                    }
                } 
                let salesThisMonth = await Saleuser.find({idUser:user.id}).populate('idUser');
                for(let sale of salesThisMonth){
                    sale.month = new Date(sale.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                    sale.year = new Date(sale.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                    if(sale.month == thisMonth && sale.year == thisYear){
                        if(sale.status == 1) user.vgv += sale.price;
                    } 
                }
                user.totalCompletion = Math.trunc((user.points / totalActivitiesPoints)*100);
                totalActivitiesPoints = 0;
            }
            salesmen.sort((a,b)=> {
               if (a.totalCompletion > b.totalCompletion) return -1;
               if (a.totalCompletion < b.totalCompletion) return 1;
               if (a.numeroPastas > b.numeroPastas) return -1;
               if (a.numeroPastas < b.numeroPastas) return 1;
               if (a.vgv > b.vgv) return -1;
               if (a.vgv < b.vgv) return 1;
            })
            res.ok(salesmen)
        }else{
            return res.status(500).send({
                success: false,
                message: "Dados não encontrados."
            });
        }
        
    },

    getSalesmenSoloGoal: async function (req, res) {
        let activities = await Activity.find();
        let thisMonth= new Date().toLocaleDateString("pt-BR").split("-")[1];
        let thisYear= new Date().toLocaleDateString("pt-BR").split("-")[0];
        let totalActivitiesPoints = 0;
        let data = {};
        data.activities = [];
        data.totalCompletion = 0;
        data.points = 0;
        let salesman = await User.findOne({id:req.user.id}).populate('idTeam');
        if(salesman){
            for(let activity of activities){
                act = {};
                act.points = 0;
                act.activityPoints = activity.score * activity.monthLimit;
                act.activity = activity;
                totalActivitiesPoints += activity.score * activity.monthLimit;
                let registersByActivity = await Activityuser.find({idUser: req.user.id, idActivity: activity.id}).populate('idUser').populate('idUserParent').populate('idActivity');
                for(let r of registersByActivity){
                    r.month = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                    r.year = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                    if(r.month == thisMonth && r.year == thisYear){
                        act.points += r.score;
                    }             
                }
                act.averageCompletion = Math.trunc((act.points / act.activityPoints)*100);
                data.activities.push(act);
            } 
            for(let a of data.activities){
                data.points += a.points;
            }
            data.totalCompletion = Math.trunc((data.points / totalActivitiesPoints)*100);
            data.totalPoints = totalActivitiesPoints;
            data.name = salesman.name;
            data.activities.sort((a,b)=> {return b.averageCompletion-a.averageCompletion;})
            let awards = await Award.find();
            salesDone = [];
            let sales = await Saleuser.find({idUser: salesman.id}).populate('idUser').populate('idUserParent').populate('idSale'); 
            for(let s of sales){
                s.month = new Date(s.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                s.year = new Date(s.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                if(s.month == thisMonth && s.year == thisYear){
                    salesDone.push(s);
                }              
            }
            maxQtd = 0;
            maxWithBonus = 0;
            maxWithoutBonus = 0;
            for(let a of awards){
                if(a.qtdSale > maxQtd){
                    maxQtd = a.qtdSale;
                    maxWithBonus = a.withBonus;
                    maxWithoutBonus = a.withoutBonus;
                }
            }
            data.salesDone = salesDone.length;
            if(data.salesDone > maxQtd){
                if(data.totalCompletion >= 80) data.bonus = maxWithBonus;
                else data.bonus = maxWithoutBonus;
            }else{
                for(let a of awards){
                    if(a.qtdSale == data.salesDone){
                        if(data.totalCompletion >= 80) data.bonus = a.withBonus;
                        else data.bonus = a.withoutBonus;
                    }
                }
            }
            
            res.ok(data)
        }else{
            return res.status(500).send({
                success: false,
                message: "Dados não encontrados."
            });
        }
        

    },

    getSalesmenGoal: async function (req, res) {
        var time = req.param('time');
        let activities = await Activity.find();
        let thisMonth= new Date().toLocaleDateString("pt-BR").split("-")[1];
        let thisYear= new Date().toLocaleDateString("pt-BR").split("-")[0];
        let totalActivitiesPoints = 0;

        if(time == 'monthly'){
            let salesmen = await User.find({idParent:req.user.id}).populate('idTeam');
            if(salesmen){
                for(let user of salesmen){
                    user.activities = [];
                    user.points = 0;
                    for(let activity of activities){
                        totalActivitiesPoints += activity.score * activity.monthLimit;
                        let registersByActivity = await Activityuser.find({idUser: user.id, idActivity: activity.id}).populate('idUser').populate('idUserParent').populate('idActivity');
                        for(let r of registersByActivity){
                            r.month = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                            r.year = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                            if(r.month == thisMonth && r.year == thisYear){
                                user.activities.push(r);
                                user.points += r.score;
                            }              
                        }
                    } 
                    user.totalCompletion = Math.trunc((user.points / totalActivitiesPoints)*100);
                    totalActivitiesPoints = 0;
                }
                salesmen.sort((a,b)=> {return b.totalCompletion-a.totalCompletion;})
                res.ok(salesmen)
            }else{
                return res.status(500).send({
                    success: false,
                    message: "Dados não encontrados."
                });
            }
        }

    },

    getTeamManagerGoal: async function (req, res) {
        var time = req.param('time');
        let activities = await Activity.find();
        let thisMonth= new Date().toLocaleDateString("pt-BR").split("-")[1];
        let thisYear= new Date().toLocaleDateString("pt-BR").split("-")[0];
        let totalActivitiesPoints = 0;


        if(time == 'monthly'){
            let teamManagers =  await User.find({idParent: req.user.id}).populate('idTeam');
            if(teamManagers){
                for(let tm of teamManagers){
                    tm.totalCompletion = 0;
                    tm.totalActivities = 0;
                    let usersByRegionalManager = await User.find({idParent:tm.id}).populate('idRole').populate('idTeam').populate('idParent');
                    
                    if(usersByRegionalManager){
                        for(let user of usersByRegionalManager){
                            user.activities = [];
                            user.points = 0;
                            for(let activity of activities){
                                totalActivitiesPoints += activity.score * activity.monthLimit;
                                let registersByActivity = await Activityuser.find({idUser: user.id, idActivity: activity.id}).populate('idUser').populate('idUserParent').populate('idActivity');
                                for(let r of registersByActivity){
                                    r.month = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                                    r.year = new Date(r.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                                    if(r.month == thisMonth && r.year == thisYear){
                                        user.activities.push(r);
                                        user.points += r.score;
                                    }              
                                }
                            } 
                            user.totalCompletion = Math.trunc((user.points / totalActivitiesPoints)*100);
                            totalActivitiesPoints = 0;
                            tm.totalActivities += user.activities.length;
                            tm.totalCompletion += user.totalCompletion;
                        }
                    }
                    tm.totalCompletion = Math.trunc(tm.totalCompletion / usersByRegionalManager.length);
                }
                teamManagers.sort((a,b)=> {return b.totalCompletion-a.totalCompletion;})
                res.ok(teamManagers);
            }else{
                return res.status(500).send({
                    success: false,
                    message: "Dados não encontrados."
                });
            }
        }

    },

    getRegionalManagerGoal: async function (req, res) {
        var time = req.param('time');
        let teams = await Team.find();
        let activities = await Activity.find();
        let thisMonth= new Date().toLocaleDateString("pt-BR").split("-")[1];
        let thisYear= new Date().toLocaleDateString("pt-BR").split("-")[0];
        let totalActivitiesPoints = 0;


        if(time == 'monthly'){
            if(teams){
                for(let t of teams){
                    t.teamAverage = 0;
                    let aux = await User.find({idTeam:t.id}).populate('idRole').populate('idTeam').populate('idParent');
                    let usersByTeam = aux.filter(element=> {
                        return element.idRole.isSalesman == 1;
                    });
                    t.members = usersByTeam;
                    if(usersByTeam){
                        for(let user of usersByTeam){
                            user.totalCompletion = 0;
                            user.points = 0;
                            for(let activity of activities){
                                totalActivitiesPoints += activity.score * activity.monthLimit;
                                let registersByActivity = await Activityuser.find({idUser: user.id, idActivity: activity.id}).populate('idUser').populate('idUserParent').populate('idActivity');
                                let filteredRegisters = registersByActivity.filter(element=>{
                                    element.month = new Date(element.createdAt).toLocaleDateString("pt-BR").split("-")[1];
                                    element.year = new Date(element.createdAt).toLocaleDateString("pt-BR").split("-")[0];
                                    return element.month == thisMonth && element.year == thisYear;
                                });
                                for(let fr of filteredRegisters) user.points += fr.score;
                            } 
                            user.totalCompletion += Math.round((user.points / totalActivitiesPoints)*100);
                            t.teamAverage += user.totalCompletion;
                            totalActivitiesPoints = 0;
                        }
                    }
                    if(t.members.length > 0) t.totalCompletion = Math.trunc(t.teamAverage / usersByTeam.length);
                }
                teams.sort((a,b)=> {return b.totalCompletion-a.totalCompletion;})
                res.ok(teams)
            }else{
                return res.status(500).send({
                    success: false,
                    message: "Dados não encontrados."
                });
            }
        }

    },


    getTeamById: async function (req, res) {
        var id = req.param('id');
      
        //Não foi enviado parâmetro
		if(!id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }

        let data = await Team.find({ id: id });
        return res.ok(data);
       
    },



    createTeam: async function (req, res) {
        let params = req.allParams();
		
		//Não foi enviado parâmetro
		if(Object.keys(params).length == 0){
			return res.status(500).send({
                    success: false,
                    message: "Não foi enviado parâmetros"
                });
		}
		
		let data = await Team.create(params).fetch();

        if (data) {
            return res.ok({
                success: true,
            });
        } else {
            return res.status(500).send({
                success: false,
                message: "Ocorreu um erro durante o cadastro"
            });
        }
    },


    
    updateTeam: async function (req, res) {

        let params = req.allParams();

        //Não foi enviado parâmetro
		if(Object.keys(params).length == 0){
			return res.status(500).send({
                    success: false,
                    message: "Não foi enviado parâmetros"
                });
		}else if(!params.id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }

        var data = await Team.updateOne({ "id": params.id }).set(params);

        if (data) {
            return res.ok({
                success: true,
            });
        } else {
            return res.status(500).send({
                success: false,
                message: "ID informado, não foi encontrado"
            });
        }
    },


    
    deleteTeamById: async function (req, res) {
        var id = req.param('id');
        //Não foi enviado parâmetro
		if(!id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }
        
        var data = await Team.destroyOne({
            id: id,
        }, function (err, success) {

            if (err) {             
                return res.status(500).send({
                    success: false,
                    message: "Não foi possível deletar",
                    err: err
                });
            } else {
                return res.ok({
                    success: true,
                });
            }
        });
    }};
