/**
* User.js
*
* @description :: user controller imported from localhost MySql server at 18/3/2020 16:24:20.
* @docs        :: http://sailsjs.org/#!documentation/controllers
* @routes      :: /* User /*
*              'get /user/getAllUser/': 'UserController.getAllUser',
*              'get /user/getUserById/:id/': 'UserController.getUserById',
*              'post /user/createUser/': 'UserController.createUser', 
*              'put /user/updateUser/': 'UserController.updateUser', 
*              'detele /user/deleteUserById/:id/': 'UserController.deleteUserById',
* 
*/


module.exports = {

    
    getAllUsers: async function (req, res) {
            let data = await User.find();
            return res.ok(data);
    },

    

    getAllRegionalManagers: async function (req, res) {
        let data = await User.find().populate('idTeam').populate('idRole');
        if(data.length > 0){
            let regionalManagers = data.filter(element=>{
                return element.idRole.isRegionalManager == 1;
            })
            res.ok(regionalManagers);
        }else{
            return res.status(500).send({
                success: false,
                message: "Nenhum usuário encontrado."
            });
        } 
    },

    getAllTeamManagers: async function (req, res) {
        let data = await User.find().populate('idRole').populate('idTeam').populate('idParent');
        if(data){
            let teamManagers = data.filter(element=>{
                return element.idRole.isTeamManager == 1;
            });

            let user = await User.findOne({id: req.user.id});

            if(user){
                let type = await Role.findOne({id: user.idRole});
                if(type.isAdmin == 1){
                    res.ok(teamManagers);
                }
                else if(type.isRegionalManager == 1){
                    let teamManagerByRegionalManager = teamManagers.filter(element =>{
                        return element.idParent.id == req.user.id;
                    });
                    res.ok(teamManagerByRegionalManager);
                }
            }
            else{
                return res.status(500).send({
                    success: false,
                    message: "Nenhum usuário encontrado."
                });
            }  
        }else{
            return res.status(500).send({
                success: false,
                message: "Nenhum usuário encontrado."
            });
        } 
    },

    getAllSalesmen: async function (req, res) {
        let data = await User.find().populate('idRole').populate('idTeam').populate('idParent');
        if(data){
            let salesmen = data.filter(element=>{
                return element.idRole.isSalesman == 1;
            });
            let user = await User.findOne({id: req.user.id});
            if(user){
                let type = await Role.findOne({id: user.idRole});
                if(type.isAdmin == 1){
                    res.ok(salesmen);
                }
                else if(type.isTeamManager == 1){
                    let salesmenByTeamManager = salesmen.filter(element =>{
                        return element.idParent.id == req.user.id;
                    });
                    res.ok(salesmenByTeamManager);
                }
            }else{
                return res.status(500).send({
                    success: false,
                    message: "Nenhum usuário encontrado."
                });
            }
        }else{
            return res.status(500).send({
                success: false,
                message: "Nenhum usuário encontrado."
            });
        } 
    },

    getUsersByTeamId: async function (req, res) {
        var id = req.param('id');
      
        //Não foi enviado parâmetro
		if(!id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }

        let data = await User.find({ idTeam: id });
        return res.ok(data);
    },

    getUsersByIdParent: async function (req, res) {
        var id = req.param('id');
      
        //Não foi enviado parâmetro
		if(!id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado"
            });
        }

        let data = await User.find({ idParent: id });
        return res.ok(data);
    },


    createRegionalManager: async function (req, res) {
        var params = req.allParams();

        if(Object.keys(params).length == 0){
			return res.status(500).send({
                success: false,
                message: "Não foi enviado parâmetros"
            });
        }

        let role = await Role.find({isRegionalManager : 1});

        let data = await User.create({
            email: params.email,
            password: params.password,
            name: params.name,
            idRole: role[0].id,
            idTeam: params.idTeam
        }).fetch();

        if(data){
            return res.ok({
                data: data,
                success: true
            });
        }else{
            return res.ok({
                success: false
            });
        }
        
    },

    createTeamManager: async function (req, res) {
        var params = req.allParams();

        if(Object.keys(params).length == 0){
			return res.status(500).send({
                success: false,
                message: "Não foi enviado parâmetros"
            });
        }
        let parent = await User.find({id: params.idParent});
        let role = await Role.find({isTeamManager : 1});

        let data = await User.create({
            email: params.email,
            password: params.password,
            name: params.name,
            idRole: role[0].id,
            idParent: params.idParent,
            idTeam: parent[0].idTeam
        }).fetch();

        if(data){
            return res.ok({
                data: data,
                success: true
            });
        }else{
            return res.ok({
                success: false
            });
        }
    },

    createSalesman: async function (req, res) {
        var params = req.allParams();

        if(Object.keys(params).length == 0){
			return res.status(500).send({
                success: false,
                message: "Não foi enviado parâmetros"
            });
        }
        let parent = await User.find({id: params.idParent});
        let role = await Role.find({isSalesman : 1});

        let data = await User.create({
            email: params.email,
            password: params.password,
            name: params.name,
            idRole: role[0].id,
            idParent: params.idParent,
            idTeam: parent[0].idTeam
        }).fetch();

        if(data){
            return res.ok({
                data: data,
                success: true
            });
        }else{
            return res.ok({
                success: false
            });
        }
    },

    updateRegionalManager: async function (req, res) {
        let params = req.allParams();
        //Não foi enviado parâmetro
		if(Object.keys(params).length == 0){
			return res.status(500).send({
                    success: false,
                    message: "Não foi enviado parâmetros."
                });
		}else if(!params.id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado."
            });
        }

        var data = await User.updateOne({ "id": params.id }).set({
            name: params.name,
            email: params.email,
            idTeam: params.idTeam
        });

        if (data) {
            if(params.password != ""){
                var changePass = await User.updateOne({ "id": params.id }).set({
                    password: params.password
                });
                if(changePass){
                    return res.ok({
                        success: true,
                    });
                }
            }else{
                return res.ok({
                    success: true,
                });
            }   
        } else {
            return res.status(500).send({
                success: false,
                message: "Usuário não encontrado."
            });
        }
    },

    deleteManager: async function (req, res) {
        var id = req.param('id');

        if(!id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado."
            });
        }

        let parent = await User.find({idParent : id});
        if(parent.length > 0){
            res.ok({
                success: false,
                message:"Existem usuários vinculados a este gerente."
            })
        }else{
            let user = await User.destroyOne({id: id});
            if(user){
                res.ok({
                    success: true
                });
            }else{
                res.ok({
                    success: false,
                    message:"Não foi possível excluir este gerente."
                });
            }
        }
    },

    updateTeamManager: async function (req, res) {
        let params = req.allParams();

		if(Object.keys(params).length == 0){
			return res.status(500).send({
                    success: false,
                    message: "Não foi enviado parâmetros."
                });
		}else if(!params.id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado."
            });
        }

        let parent = await User.find({id: params.idParent});

        let user = await User.findOne({ "id": params.id });


        var data = await User.updateOne({ "id": params.id }).set({
            name: params.name,
            email: params.email,
            idParent: params.idParent,
            idTeam: parent[0].idTeam
        });

        if (data) {
            let updatedSalesmen = await User.update({ "idParent": user.id }).set({
                idTeam: parent[0].idTeam
            });
            if(params.password != ""){
                var changePass = await User.updateOne({ "id": params.id }).set({
                    password: params.password
                });
            }  
            res.ok({
                success: true
            })

        } else {
            return res.status(500).send({
                success: false,
                message: "Usuário não encontrado."
            });
        }
    },

    updateSalesman: async function (req, res) {
        let params = req.allParams();

        //Não foi enviado parâmetro
		if(Object.keys(params).length == 0){
			return res.status(500).send({
                    success: false,
                    message: "Não foi enviado parâmetros."
                });
		}else if(!params.id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado."
            });
        }

        let parent = await User.find({id: params.idParent});

        var data = await User.updateOne({ "id": params.id }).set({
            name: params.name,
            email: params.email,
            idParent: params.idParent,
            idTeam: parent[0].idTeam,
            password: params.password
        });

        if (data) {
            if(params.password != ""){
                var changePass = await User.updateOne({ "id": params.id }).set({
                    password: params.password
                });
                if(changePass){
                    return res.ok({
                        success: true,
                    });
                }
            }else{
                return res.ok({
                    success: true,
                });
            }   
        } else {
            return res.status(500).send({
                success: false,
                message: "Usuário não encontrado."
            });
        }
    },

    deleteSalesman: async function (req, res) {
        var id = req.param('id');

        if(!id){
            return res.status(500).send({
                success: false,
                message: "O id não foi enviado."
            });
        }
        let user = await User.destroyOne({id: id});
        if(user){
            res.ok({
                success: true
            });
        }else{
            res.ok({
                success: false,
                message:"Não foi possível excluir este vendedor."
            });
        }
        
    },

    changePassByUser: async function (req, res) {
        let data = req.allParams();
        var user = await User.updateOne({ "id": req.user.id }).set({
            password: data.password,
        });
        if (!user) {
            return res.ok({
                success: false,
                message: "Erro ao atualizar a senha."
            });
        } else {
            return res.ok({
                success: true,
            });
        }
    },

    userInfo: async function (req, res) {
        const user = await User.findOne({id:req.user.id}).populate('idParent').populate('idTeam');
        if (!user) {
            return res.ok({
                success: false,
                message: "Usuário não encontrado"
            });
        } else {
            return res.ok(user);
        }
    },

    uploadAvatar: async function (req, res) {
        var uploadFile = req.file('file');
        let path = await sails.helpers.uploadImage.with({
            path: '../../assets/files/userProfiles/',
            file: uploadFile
        });
        if (path) {
                User.updateOne({ "id": req.user.id }).set({
                    profileImage: path
                }).exec(function (err) {
                    if (err) {
                        return res.ok({
                            success: false,
                            message: 'Ocorreu um erro ao realizar o upload do arquivo.'
                        });
                    }
                    return res.ok({
                        success: true
                    });
                });
        } else {
            return res.ok({
                success: false,
                message: 'Ocorreu um erro ao realizar o upload do arquivo.'
            });
        }
    },

}
